@@ -1,5 +1,5 @@
 # Sketch - A Python-based interactive drawing program
-# Copyright (C) 1996, 1997, 1998, 1999, 2000, 2001 by Bernhard Herzog
+# Copyright (C) 1996, 1997, 1998, 1999, 2000, 2001, 2004 by Bernhard Herzog
 #
 # This library is free software; you can redistribute it and/or
 # modify it under the terms of the GNU Library General Public
@@ -486,6 +486,10 @@ class SketchCanvas(SketchView, CursorStack, WidgetWithModes):
 		self.popup_context_menu(event.x_root, event.y_root,
 					button, event.state)
 		self.ignore_key_press_events = 0
+            elif button == const.Button4:
+                self.ScrollYUnits(-3)
+            elif button == const.Button5:
+                self.ScrollYUnits(3)
 	finally:
 	    self.end_transaction()
 
@@ -841,11 +845,13 @@ class SketchCanvas(SketchView, CursorStack, WidgetWithModes):
 	    self.do_end_creation(set_mode = set_mode)
 
     def creation_cancel(self):
-	if self.current is not None:
-	    self.current.Hide(self.invgc)
-	    self.current = None
-	self.create_creator = None
-	self.SelectionMode()
+        self.dragging = 0
+        self.start_drag = 0
+        if self.current is not None:
+            self.current.Hide(self.invgc)
+            self.current = None
+        self.create_creator = None
+        self.SelectionMode()
 
 
     #
